import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Color;

/**
 * Cutline is a new class that takes a line of text and can
 * display it in a box, with the text separated over up to five
 * different lines. The font, font size, and font style are
 * configurable, as are the coordinates of the box and its
 * width. Copyright 1996 Rogers Cadenhead. You can use this
 * applet freely on your own web pages, and use any part of it
 * in your own Java programming with no restrictions.
 * @author Rogers Cadenhead
 * @version 1.0
 */
public class Cutline extends Rectangle {
    String[] lines = new String[6];
    Image image;
    Graphics g;
    String text;
    int numLines;
    Font font;
    int boxWidth;
    int boxHeight;

    /**
     * This method is used to find out how tall a cutline box will
     * be for a string.
     * @param g the cutline's Graphics object
     * @param text the cutline String
     * @param f the Font
     * @param w the width of the box (int)
     * @return an integer representing the height 
    */  
    public int findHeight(Graphics g, String text, Font f, int w) {
        int n = buildBox(g, text, f, w);
        return boxHeight;
    }

    /**
     * This method is used to take a text String and break it into
     * individual lines that can be displayed in a cutline box.
     * The number of lines is limited to 5.
     * @param g the cutline's Graphics object
     * @param text the cutline String
     * @param f the Font
     * @param w the width of the box (int)
     * @return the number of lines of the divided text 
    */  
    private int buildBox(Graphics g, String text, Font f, int w) {
        // set up text
        this.text = text.trim();
        this.font = f;
        this.boxWidth = w;
        FontMetrics fm = g.getFontMetrics(f);
                       
        numLines = 0;
        lines[0] = "";
        
        // build lines of text for box
        out:
        for (int i = 0; i <= text.length(); i++) {
            String c = "";
            try { c = text.substring(i, i+1); }
            catch (StringIndexOutOfBoundsException e) { }
            lines[numLines] += c;
            if (fm.stringWidth(lines[numLines]) >= (w-5)) {
                int ls = lines[numLines].lastIndexOf(" ");
                String nextLine = "";
                if (ls != -1) {
                    try {
                        nextLine = lines[numLines].substring(ls+1);
                        lines[numLines] = lines[numLines].substring(0, ls);
                        nextLine.trim();
                    } catch (StringIndexOutOfBoundsException e) { }
                }
                numLines += 1;
                if (numLines > 5) {
                    numLines = 5;
                    break out;            
                }
                lines[numLines] = nextLine;
            }
            // set box dimensions, location
            this.boxHeight = fm.getHeight() * (numLines + 2);
        }
    	  return numLines;
    }

    /**
     * This method is used to draw a cutline in a box atop the
     * current image
     * @param g the cutline's Graphics object
     * @param xpos the x position of the box
     * @param ypos the y position of the box
     * @param text the cutline String
     * @param f the Font
     * @param w the width of the box (int)
    */  

    public void draw(Graphics g, int xpos, int ypos,
        String text, Font f, int w) {
        
        FontMetrics fm = g.getFontMetrics(f);

        // Set up text in box
        numLines = buildBox(g, text, f, w);

        // draw box
        g.setColor(Color.white);
        g.fillRect(xpos, ypos, boxWidth, boxHeight);
        g.setColor(Color.black);
        g.drawRect(xpos, ypos, boxWidth, boxHeight);
        g.setFont(f);
        for (int i = 0; i <= numLines; i++) {
            g.drawString(lines[i], xpos + 2, ypos + ((i+1) * fm.getHeight() + 2));
        }
    }

}

